%%   demo_Compressive_Sensing_Ghost_Imaging.m

%%   Demo for the paper:
%%   V. Katkovnik, J. Astola
%%  "Compressive sensing computational ghost imaging", Submitted to JASA A

%%  Katkovnik V., May 2012
close all
clear all
'START'


'Start: demo_Compressive_Sensing_Ghost_Imaging.m'

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% SMALL TEST-IMAGES %%%%%%%%%%%%%%%%%%%%%

%   load lena_small_64_64
%   y_abs=lena_small_64_64;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   load y_lena_small
%   y_abs=y_lena_small/210;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  load y_cameraman_small
%  y_abs=y_cameraman_small/242;  % image normalization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   load y_TUT_small
  y_abs=1-y_TUT_small+.1;

  %% TRUE AMPLITUDE OBJECT
u0=y_abs;  % Squared magnitude of the object transmittance

[yN,xN]=size(u0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%% PARAMETERS OF THE ALGORITHM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
IterNumber=100;     %% Number of Iterations
 
%% OPTIMAL  PARAMETERS: ''threshold_abs;" and ''gamma_00'' are given 
%% in the tables "threshold_ampl_grid_TABLE " and "gamma_00_grid_TABLE " 

%% see the files "results .... .mat" for the corresponding values of KAPPA (NOISE LEVEL PARAMETER)

%% COLUMNS and ROWS in these tables are as in TABLES in the paper;

%% FOR ''TUT'' the columns are : mumu_set =[  1     2     4     8    16]- compression ratios;

%% the rows are: d_factor_set =[ 1     3     5    10], factors for f, defining the free space propagation distance;

%% FOR cameraman: mumu_set =[ 2     4     8    16] and d_factor_set =[ 1     2     3     5    10]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
  %% for TUT test-image (KAPPA=10000,  d_factor=3 for free propagation distance, mu=2)
  d_factor=3; mu=2;
   threshold_ampl=.05; sigma=0.; gamma_00=.0002; % PSNR=45.82 dB

    %% for Cameraman test-image (KAPPA=10000,  d_factor=1 for free propagation distance, mu=2)
%   d_factor=1; mu=2; ss00=50 !!!!!
%   threshold_ampl=.05; sigma=0.; gamma_00=.04; % PSNR= 28.51 dB
 %% BM3D-frame PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Nstep = 1;  N11=4;  N22=8;

threshType='h';  %% 'h' for hard-thresholding; 's' for soft-thresholding
                        %%% hard-thresholding is recommended
                 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% CONTROL ALGORITHM PARAMETERS

filtering=1;       %% filtering=1 switches on BM3D filtering; filtering=0 switches of BM3D filtering
modulation_type=1; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PARAMETERS of 4f  SETUP 
lambda = 532e-6;                             %% wavelength [mm]
%delta = 3.45e-3;                              %% pixel size for sensor, in mm
%delta = 20e-3;                              %% pixel size for SLM, in mm
delta = 8e-3;                                  %% pixel size for SLM, in mm

%% in-focuss distance
d_f=delta*delta*yN/lambda; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

d=d_f*d_factor;                                           %% free propagation distance;
%mu=2;                        %% compressive sampling ratio
K=floor(yN*xN/mu);        %% number of experiments
KAPPA=10000;              %% parameter of Poissonian observations
 


%% INITIALIZATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

av=1;                        % averaging for F-DDT foreard propagation
Model_Up_Date=1;    %% 1 for new observation-data,  0 for the latest saved observation-data

Gauss_var=0;            %% '1' for the varying weights and '0' for the invariant weights of approximative Gaussian distribution

IdealWeights=1;        %% Ideal varying weights for Gaussian distribution with the varying variances; THIS PARAMETER WORKS only for Gauss_var=1;
IdealWeights=0;         %% Adaptive weights for var GAUSS; THIS PARAMETER WORKS only for Gauss_var=1


ss00=floor(IterNumber/4);  %% update FRAMES and go to varying Gaussian estimation

%ss00=50; for cameraman
%% SLM MASK and OBSRVATION GENERATOR %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


 
if Model_Up_Date==1   %% 1 switches to RAND MASK, , '0' switches to the saved MASKS
'Please wait: Generation of Poissonian Observations'
B=[];

rand('seed',500001);
    for s_r=1:K                       %% Loop over the set of observation experiments
                                           
        MASK=exp(j*2*(pi-.01)*rand(yN,xN));                         %% RANDOM PHASE SLM MASK 
      
        
        [u_r]=FresnelTransform(MASK,d,lambda,delta,av);      %% F-DDT forward propagation
    
        
    % function [A,cond_A,a_ks] = function_TransferFunction_FDDT(z,lambda,delta,N,M,av)
      
         b_r=(abs(u_r(:))).^2;
         x_r=b_r.*abs(u0(:));                                                %% noiseless intensity wave field
         
        %% POISSONIAN NOISY DATA %%%%%%%%%%
         rand('seed',s_r+100001), randn('seed',s_r+1001);        %% seeds, fixing the Poissonian random observations
        zz(s_r)=poissrnd(KAPPA*sum(abs(x_r(:))))/KAPPA;   %% Poissonian data
         
        z_noiseless(s_r)=sum(abs(x_r(:)));                            %% noiseless data
      
     
         B=[B b_r];
               
    end
  
   ssigma_2=(mean(zz(:))/KAPPA);  % estimate of variance
   gamma_0=gamma_00/ssigma_2;  % parameter o the algorithm
   zz=zz';
  
   FHI=pinv(B'*B+eye(K)/gamma_0);
   delta_x_zz=FHI*(B'*B)*zz;
   
             
   save AB B zz ssigma_2 z_noiseless 
   
    
else
    'Using Old Poissonian Observations'
 load AB 
 gamma_0=gamma_00/ssigma_2; 
 
 B=B(:,1:K); 
 zz=zz(1:K);

 z_noiseless=z_noiseless(1:K);
 FHI=pinv(B'*B+eye(K)/gamma_0);
 delta_x_zz=FHI*(B'*B)*zz;
   
 

end  % END for 'Model_Up_Date'
tic

mean_B=mean(B');   %% for sigma^2 update

%% Approximative Gaussian ALGORITHM, CSGI_inv and CSGI_var

 v0=ones(yN,xN)/2;          %% Intialization 


for ss=[1:IterNumber]                           %% Algorithm's iterations
    
     
      
   if (Gauss_var==1)&(ss>=ss00)           %% START CSGI-varying (variang variance)
        if (ss==ss00)&(IdealWeights==1)
            'Ideal Weights'
    D=diag(1./z_noiseless*KAPPA)*ssigma_2;
    FHI=pinv(B'*B*D+eye(K)/gamma_0);
     
    delta_x_zz=FHI*(B'*B)*D*zz;
        end
      %% CSGI_var
 if (ss>=ss00)&(IdealWeights==0)
   
       for ss1=1:K
      z_noiseless(ss1)=sum(B(:,ss1).*abs(u0_est(:)));  
       end 
    D=diag(1./z_noiseless*KAPPA)*ssigma_2;
    FHI=pinv(B'*B*D+eye(K)/gamma_0);
     
    delta_x_zz=FHI*(B'*B)*D*zz;
 end
   %% end of CSGI_var
   
   x=delta_x_zz+FHI*B'*v0(:)/gamma_0;
    
   u0_est=v0(:)+B*D*(zz-x)*gamma_0;
   

   else 
          %% CSGI_inv s
   x=delta_x_zz+FHI*B'*v0(:)/gamma_0;
    
   u0_est=v0(:)+B*(zz-x)*gamma_0; %0*PSI_PSI*rand(yN*yN,1)/(ss);
   
 %% variance update ( can be dropped) 
   ssigma_22(ss)=mean(B')*abs(u0_est)/KAPPA;
   gamma_0=gamma_00/ssigma_2;
 
    end
       
  
    
   
  
u0_est=reshape(u0_est,[yN,xN]); 
u0_est=abs(u0_est).*double((real(u0_est)>=0));

%% RESULTS of FIRST STEP 
ee_abs=u0_est-abs(u0); 

rmse_abs(ss)=sqrt(mean(ee_abs(:).^2));

 %% BM3D-frame filtering
 %keyboard
 %% FILTERING STARS from ITERATION ss=1
if (ss>=1)&(ss<ss00)  
 
  kk=10;  u0_est_filt=padarray((u0_est), [kk kk],'replicate', 'both'); %% PADDING FOR FILTERING, can be k=0, NO PADDING;
      
 [v0tmp1] = BM3D_filter(u0_est_filt,threshType, threshold_ampl, N11, N22, Nstep,filtering,ss,1);
 
 v0=v0tmp1(kk+1:end-kk,kk+1:end-kk);
  
else
  %% Frame update for ss=ss00
  kk=10;  u0_est_filt=padarray((u0_est), [kk kk],'replicate', 'both'); %% PADDING FOR FILTERING, can be k=0, NO PADDING;
      
 [v0tmp1] = BM3D_filter(u0_est_filt,threshType, threshold_ampl, N11, N22, Nstep,filtering,ss,ss00); 
 
 
 v0=v0tmp1(kk+1:end-kk,kk+1:end-kk);
  
  
 % u0_est=v0;
end  


PSNR(ss)=20*log10(max(y_abs(:))/rmse_abs(ss));

'[PSNR, rmse]', [ss PSNR(ss) rmse_abs(ss)]

%% ITERATION VISUALIZATION

figure(2), subplot(2,2,1),imshow(abs(u0_est),[]), title(['CSGI, RECONST, PSNR = ' num2str(PSNR(ss),3)]), 
subplot(2,2,2),imshow(abs(u0),[]),        title(['TRUE IMAGE']),...
subplot(2,2,4), plot(PSNR), title('PSNR, dB'),grid on, xlabel('ITER NUMBER'),...

subplot(2,2,3), plot(abs(u0(7,:)),'r'), hold on, plot(abs(u0_est(7,:))), title(['CROSS-SEC']), hold off
h=figure(2);


%F(ss)=getframe(h);               % frames for avi-video




end %% END of MAIN ALGORITHM, ss
 
toc



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 % movie2avi(F,'CC','fps',5,'quality',100) ;  % avi movies of iterations
