%% demo_GHOST_CORRELATION_EST.m
%%   Demo for the paper:
%%   V. Katkovnik, J. Astola
%%  Compressive sensing computational ghost imaging, Submitted to JASA A
%%  For the cross-correlation GI we use the novel estimate derived in this paper

%%  Katkovnik V., March 2012

close all
clear all
'START'
tic

'Start: demo_GHOST_CORRELATION_EST.m'

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% SMALL TEST-IMAGES %%%%%%%%%%%%%%%%%%%%%

%    load lena_small_64_64
%    y_abs=lena_small_64_64;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    load y_lena_small
%    y_abs=y_lena_small/210;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    load y_cameraman_small
%    y_abs=y_cameraman_small/242;  % image normalization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       load y_TUT_small
       y_abs=1-y_TUT_small+.1;

  %% TRUE AMPLITUDE OBJECT
u0=y_abs;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[yN,xN]=size(u0);
lambda = 532e-6;                           %% wavelength [mm]
%delta = 3.45e-3;                           %% pixel size for sensor, in mm
%delta = 20e-3;                              %% pixel size for SLM, in mm
delta = 8e-3;                                  %% pixel size for SLM, in mm

%% in-focuss distance
f=delta*delta*yN/lambda; 

d_factor=3;
d=f*d_factor;    %% free propagation distance;
av=1;       

KAPPA=10000; %% Poissonian Noise Parameter

K=30000;            %% Number of Experiments
rand('seed',500001);
 %% Main Loop for Experimental Data Simulation and Reconstruction
    for s_r=1:K                                                %% Loop over the set of observation experiments
                                        
        MASK=exp(j*2*(pi-.01)*rand(yN,xN));      %% RANDOM PHASE SLM MASK 
  
        [u_r]=FresnelTransform(MASK,d,lambda,delta,av); %% F-DDT forward propagation
     
         b_r=(abs(u_r(:))).^2;
         
         x_r=b_r.*abs(u0(:));
         
        
         %%%%%%%%% POISSONIAN NOISY DATA %%%%%%%%%%
         rand('seed',s_r+100001), randn('seed',s_r+1001);
         
        zz(s_r)=poissrnd(KAPPA*sum(abs(x_r(:))))/KAPPA;  %% Observations
         
        z_noiseless(s_r)=sum(abs(x_r(:)));
      
        
      
      if s_r==1
          S_zz=zz(s_r);
          S_b_r=b_r;
          delta_b_r=ones(yN*xN,yN*xN);
          S=(zz(s_r)- S_zz)*(b_r-S_b_r);

      else
          
       S_zz=S_zz-(S_zz-zz(s_r))/(s_r);  %% mean
       S_b_r=S_b_r-(S_b_r-b_r)/(s_r); %% mean
       
       delta_b_r=delta_b_r-(delta_b_r-(b_r-S_b_r)*(b_r-S_b_r)')/(s_r); % mean
       
      S=S-(S-(zz(s_r)- S_zz)*(b_r-S_b_r))/(s_r);
   
      end
      
     
     u0_est=reshape((delta_b_r+eye(yN*xN)*.01)\S,[yN,xN]);
     
 
      ee_abs=u0_est-abs(u0); 

rmse_abs(s_r)=sqrt(mean(ee_abs(:).^2));
PSNR(s_r)=20*log10(max(y_abs(:))/rmse_abs(s_r));

%% Result Visualization
%  figure(11), subplot(1,2,1), imshow( u0_est,[]), title('CROSS-CORRELATION  GI'), ...
%       subplot(1,2,2), plot(PSNR(:)), title('PSNR,  dB'),  xlabel('Experiment Number')
             figure(11), subplot(2,2,1), imshow( u0_est,[]), title('CROSS-CORRELATION  GI'), ...
      subplot(2,2,2), plot(PSNR(:)), title('PSNR,  dB'),  xlabel('Experiment Number'),...
      subplot(2,2,3), plot(abs(u0(7,:)),'r'), hold on, plot(abs(u0_est(7,:))), title(['CROSS-SEC OF RECONST']), hold off
      
      
    end
toc
